/* SPDX-License-Identifier: MIT */
#ifndef __NVKM_GRAPH_OS_H__
#define __NVKM_GRAPH_OS_H__

#define E_BAD_COMMAND  0x00000001
#define E_CMD_OVERFLOW 0x00000002
#define E_BAD_FWMTHD   0x00000003

#endif
