// SPDX-License-Identifier: GPL-2.0
// Copyright(c) 2018 Intel Corporation. All rights reserved.

#include <linux/module.h>
#include <linux/printk.h>
#include "watermark.h"

nfit_test_watermark(device_dax);
